package jp.co.hogehoge.system;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

// commons
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.util.text.Crypt;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.io.ScreenLockIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;


/**
 * 【Example】ユーザー設定画面。
 * <br>
 * <br>Copyright(c) 1999-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class ConfigEvent
    extends AbstractExampleScreenEvent
{
    /** Serial */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        if (!super.isPostBack()) {
        // 他画面からの遷移
    
            if (super.isInitialize()) {
            // 初期状態

                // [仕様]初期遷移時は照会モードで開く
                this.btn_mode_view_event();
            }
        }
    }


    /**
     * 照会モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_view_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();
        ctrl.setReadOnlyModeAll(ScreenLockIF.READONLY.READONLY_LABEL);
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE,           "BTN_MODE_EDIT");

        this.find();
        super.saved();

        this.rewrite("表示モードの書換え");
    }

    /**
     * 編集モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_edit_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_EDIT");

        // [仕様]新規登録機能なし(常に既存データ)
        this.find();
        super.saved();

        this.rewrite("表示モードの書換え");
    }

    /**
     * 保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        // ========================================================================================
        // 所属部署の優先順位変更
        // ========================================================================================

        // リスト
        final ListBeanIF list = super.getSubList("GROUP_LIST");

        // 行データ
        ListBeanIF.SubListBeanIF sub;

        if (list != null && list.getSize() != 0) {

            list.sort(new String[]{"VIEWLINE"});

            for (int i = 0; i < list.getSize(); i++)
            {
                sub = list.get(i);
                sub.setInteger("VIEWLINE", i);
            }
        }

        // ========================================================================================
        // 入力チェック：保存処理用
        // ========================================================================================
        return super.getRuleEngine("Save").check();
    }

    /**
     * 保存処理。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_event()
        throws RuleException
    {
        // ルール・エンジン：保存処理
        final RuleEngineIF rule = super.getRuleEngine("Save");

        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        try {

            // ====================================================================================
            // 事前処理
            // ====================================================================================
            // ------------------------------------------------------------------------------------
            // パスワードの変更準備
            // ------------------------------------------------------------------------------------
            if (super.getString("PASSWORD") != null) {
            // パスワードの変更あり

                rule.getParameter().put("CRYPT", Crypt.toHash(super.getString("PASSWORD")));
            }

            // ====================================================================================
            // 変更処理
            // ====================================================================================
            if (rule.run()) {
            // 保存処理:成功

                // --------------------------------------------------------------------------------
                // ログイン認証情報を最新の状態に更新する
                // --------------------------------------------------------------------------------

                // 最新の状態に更新する
                info.setUserName(super.getString("USER_NAME"));
                info.setUserEMail(super.getString("USER_EMAIL"));
                info.setUserPhone(super.getString("USER_PHONE"));

                // 部門一覧(ユーザー情報)
                final List<Map<String, Object>> groupList
                 = new ArrayList<>();

                for (final ListBeanIF.SubListBeanIF tmp : super.getSubList("TMP_GROUP_LIST").getList())
                {
                    groupList.add(tmp.getAll());
                }
                info.setUserGroupAuthority(groupList);
                info.setCurrentGroupAuthority(0);

                // --------------------------------------------------------------------------------
                // 画面再描画(照会)
                // --------------------------------------------------------------------------------
                this.btn_mode_view_event();

                return true;
    
            } else {
            // 保存処理:失敗

                return false;
            }

        } finally {

            // ====================================================================================
            // 事後処理
            // ====================================================================================
            // 一時保存データの消去
            super.setString("CRYPT",              null);
            super.setString("TMP_GROUP_LIST",     null);
        }
    }


    /**
     * 検索処理を実行。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    private void find()
        throws RuleException
    {
        super.getRuleEngine("Find.Detail").run();
    }

    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "表示モードの書換え":
            {
                ctrl.addRewrite("BLOCK_BUTTON_CTRL1");
                ctrl.addRewrite("BLOCK_DETAIL");
                break;
            }
            case "全部書換え":
            {
                ctrl.clearRewrite();
                ctrl.addRewrite("ALL");
            }
        }
    }
}
