package jp.co.hogehoge.system;

import java.sql.Date;
import java.util.Calendar;
import java.util.List;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.USAXException;
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF.SubListBeanIF;
import jp.co.cam.usax.rule.bean.MessageIF;
import jp.co.cam.usax.rule.bean.impl.ListBean;
import jp.co.cam.usax.rule.bean.resource.AttributeResourceIF;
import jp.co.cam.usax.rule.bean.resource.AttributeResourceIF.AttributeIF;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.util.text.format.datetime.DateTimeFactory;
import jp.co.cam.usax.util.text.format.datetime.DateTimeIF;
import jp.co.cam.usax.util.text.format.datetime.festival.FestivalManagerIF;
import jp.co.cam.usax.util.text.format.datetime.festival.FestivalManagerIF.FestivalIF;
// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.system.help.HELP0000SEvent;


/**
 * 【Example】定休日設定画面。
 * <br>各法人の定休日設定を行う画面。
 * <br>
 * <br>[関連画面]
 * <br>SYS1300S 休日設定画面
 * <br>SYS1300D 定休日設定画面(this)
 * <br>
 * <br>[TABLE]
 * <br>SYS_CORPORATE_HOLIDAY    C 法人.休日テーブル
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS1300DEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        if (super.isPostBack()) {

            log.debug("============================================================");
            log.debug("(B)処理後の画面再表示");
            log.debug("============================================================");

        } else {

            log.debug("============================================================");
            log.debug("他画面からの遷移");
            log.debug("遷移元画面", super.getInitialBackPage());
            log.debug("============================================================");
            if (super.isInitialize()) {

                log.debug("------------------------------------------------------------");
                log.debug("1.初期処理");
                log.debug("1-1.法人情報の取得");
                log.debug("------------------------------------------------------------");

                // ルールエンジン:初期処理
                final RuleEngineIF initializeRule = super.getRuleEngine("Initialize");

                initializeRule.run();

                log.debug("------------------------------------------------------------");
                log.debug("1-2.初期値の設定:6週間データ");
                log.debug("------------------------------------------------------------");

                // 月間定休日データ
                final ListBeanIF monthly = new ListBean(this);

                // 週間データ
                SubListBeanIF week;

                for (int i = 0; i < 6; i++)
                {
                    week = monthly.createSubBean();
                    week.setString("SUN", "1");
                    week.setString("SAT", "1");
                    monthly.add(week);
                }
                super.setSubList("MONTHLY", monthly);

                log.debug("------------------------------------------------------------");
                log.debug("1-3.初期値の設定:年");
                log.debug("------------------------------------------------------------");

                // カレンダーAPI
                final Calendar calendar = Calendar.getInstance();

                // 現在年
                final int year = calendar.get(Calendar.YEAR);

                log.debug("現在年", year);

                super.setInteger("YEAR", year);
            }
        }
    }


    /**
     * 休日の登録処理に係る入力チェック。
     * <br>1.画面の入力値の妥当性をチェック
     * <br>1-1.処理対象年の必須入力チェック
     * <br>1-2.処理対象年の範囲チェック
     * @return true 正常 or false 異常
     * @throws RuleException 処理中に発生した例外を報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("1.画面で入力された処理対象年の妥当性をチェック");
        log.debug("1-1.処理対象年の必須入力チェック");
        log.debug("1-2.処理対象年の範囲チェック");
        log.debug("============================================================");

        try {

            // 処理対象年
            final int year = super.getInteger("YEAR");

            if (year < 2000 || 2150 < year) {
                throw new IllegalArgumentException("処理対象年に不正な入力値 - " + year);
            }

        } catch (final Exception ex) {

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            msg.addMessage("{@attribute:ERR_YEAR}");
            msg.addField("YEAR");

            return false;
        }
        return true;
    }

    /**
     * 休日の登録処理。
     * <br>1.選択された定休日を休日として登録する
     * <br>2.祝日を休日として登録する
     * <br>3.処理完了を出力する
     * @throws RuleException 処理中に発生した例外を報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        // カレンダーAPI
        final Calendar calendar = Calendar.getInstance();

        // 月間定休日データ
        final ListBeanIF monthly;

        // 週間データ
        SubListBeanIF week;

        // 処理年
        int year;

        // 曜日
        int dayOfWeek;

        // 週
        int weekLine;

        monthly = super.getSubList("monthly");
        year = super.getInteger("YEAR");

        log.debug("============================================================");
        log.debug("1.選択された定休日を休日として登録する", super.getInteger("YEAR") +"年");
        log.debug("============================================================");

        calendar.set(Calendar.YEAR, year);

        for (int month = 0; month < 12; month++)
        {
            log.debug("------------------------------------------------------------");
            log.debug((month + 1) +"月");
            log.debug("------------------------------------------------------------");
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.set(Calendar.MONTH, month);
            weekLine = 0;

            for (int day = 0; day < 31; day++)
            {
                log.debug((day + 1) +"日");

                switch (calendar.get(Calendar.DAY_OF_WEEK))
                {
                    case Calendar.MONDAY:
                    {
                        dayOfWeek = 0;
                        break;
                    }
                    case Calendar.TUESDAY:
                    {
                        dayOfWeek = 1;
                        break;
                    }
                    case Calendar.WEDNESDAY:
                    {
                        dayOfWeek = 2;
                        break;
                    }
                    case Calendar.THURSDAY:
                    {
                        dayOfWeek = 3;
                        break;
                    }
                    case Calendar.FRIDAY:
                    {
                        dayOfWeek = 4;
                        break;
                    }
                    case Calendar.SATURDAY:
                    {
                        dayOfWeek = 5;
                        break;
                    }
                    case Calendar.SUNDAY:
                    {
                        dayOfWeek = 6;

                        if (day != 0) {
                            log.debug("翌週へ");
                            weekLine++;
                        }
                        break;
                    }
                    default:
                    {
                        dayOfWeek = -1;
                    }
                }

                week = monthly.get(weekLine);

                if ("1".equals(week.getString(HELP0000SEvent.WEEKS[dayOfWeek]))) {
                    createHoliday(this.getHolidayType(0), new Date(calendar.getTime().getTime()), 0);
                }
                calendar.add(Calendar.DAY_OF_MONTH, 1);

                if (month < calendar.get(Calendar.MONTH)) {
                    log.debug("次月へ");
                    break;
                }
            }
        }

        log.debug("============================================================");
        log.debug("2.祝日を休日として登録する(振替休日・国民の休日を含む)");
        log.debug("============================================================");
        if ("1".equals(super.getString("FESTIVAL"))) {

            // ユーザー情報
            final UserInfo info = UserInfo.getUserInfo();

            // 祝日管理クラス
            final FestivalManagerIF festivalManager;

            // 日時API
            final DateTimeIF dateTime;

            // 祝日一覧
            final List<FestivalIF> festivalList;

            log.debug("祝日の休日設定", "あり");

            try {

                dateTime = DateTimeFactory.get(info.getLanguage(), info.getCountry());

                festivalManager = dateTime.getFestivalManager();

                festivalList = festivalManager.getFestivalList(year);

            } catch (final USAXException ex) {
                throw new RuleException(ex, "祝日情報の取得に失敗(祝日設定を確認してください)");
            }

            // 休日
            java.util.Date holiday;

            for (final FestivalIF festival : festivalList)
            {
                holiday = festival.getFestivalDate();

                if (holiday != null) {
                    this.createHoliday(festival.getFestivalName(), new Date(holiday.getTime()), 9);
                }

                holiday = festival.getSubstitute();

                if (holiday != null) {
                    this.createHoliday(this.getSubstitute(), new Date(holiday.getTime()), 9);
                }
            }

        } else {
            log.debug("祝日の休日設定", "なし");
        }

        log.debug("============================================================");
        log.debug("3.処理完了を出力する");
        log.debug("============================================================");

        // メッセージ・コンテナ
        final MessageIF msg = super.getMessage();

        msg.addMessage("{@message:COMPLETED}");
    }

    /**
     * 休日区分の名称を取得。
     * @param  pmType 休日区分
     * @return 休日区分の名称
     * @throws RuleException Attribute定義に異常がある場合
     */
    private String getHolidayType(
        final int pmType
    )
        throws RuleException
    {
        // Attribute定義一覧
        final AttributeResourceIF resource = super.getAttributeResource();

        if (resource.hasAttribute("HOLIDAY_TYPE")) {

            // Attribute定義
            final AttributeIF attr = resource.getAttribute("HOLIDAY_TYPE");
    
            return attr.getLabel(String.valueOf(pmType));

        } else {
            throw new RuleException("Attribute定義を確認してください - HOLIDAY_TYPE");
        }
    }

    /**
     * 振替休日の名称を取得。
     * @return 振替休日の名称
     * @throws RuleException Attribute定義に異常がある場合
     */
    private String getSubstitute()
        throws RuleException
    {
        // Attribute定義一覧
        final AttributeResourceIF resource = super.getAttributeResource();

        if (resource.hasAttribute("HOLIDAY_TYPE")) {

            // Attribute定義
            final AttributeIF attr = resource.getAttribute("SUBSTITUTE");

            return attr.getName();

        } else {
            throw new RuleException("Attribute定義を確認してください - SUBSTITUTE");
        } 
    }

    /**
     * 休日の登録。
     * <br>既存の休日設定があれば削除して登録します。
     * <br>1.指定された日の休日設定があれば削除
     * <br>2.指定された休日を登録
     */
    private void createHoliday(
        final String pmName
      , final Date pmDate
      , final int pmType
    )
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("------------------------------------------------------------");
        log.debug(pmName, pmDate);
        log.debug("------------------------------------------------------------");

        // ルールエンジン:休業日の登録処理
        final RuleEngineIF createHolidayRule = super.getRuleEngine("CreateHoliday");

        try {

            super.setString("HOLIDAY_NAME", pmName);
            super.setDate("HOLIDAY_DATE", pmDate);
            super.setInteger("HOLIDAY_TYPE", pmType);
    
            if (createHolidayRule.check()) {
                log.debug("1.指定された日の休日設定があれば削除");
                log.debug("2.指定された休日を登録");
                createHolidayRule.run();
            }

        } finally {

            super.setString("HOLIDAY_NAME", null);
            super.setString("HOLIDAY_DATE", null);
            super.setString("HOLIDAY_TYPE", null);
        }
    }
}
