package jp.co.hogehoge.system;

import java.sql.Date;

// commons
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.impl.extra.PaginationBean;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.EventIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;


/**
 * 【Example】休日設定画面(検索/一覧)。
 * <br>各法人の休日(定休日/休業日/祝日・休日)設定を行う画面。
 * <br>
 * <br>[関連画面]
 * <br>SYS1300S 休日設定画面(this)
 * <br>SYS1300D 定休日設定画面
 * <br>
 * <br>[TABLE]
 * <br>SYS_CORPORATE            R     法人一覧の取得(システム管理者)
 * <br>SYS_CORPORATE_HOLIDAY    C/R/D 法人.休日テーブル
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS1300SEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        if (super.isPostBack()) {

            log.debug("============================================================");
            log.debug("(B)処理後の画面再表示");
            log.debug("============================================================");

        } else {

            log.debug("============================================================");
            log.debug("(A)メニュー及び他画面からの遷移");
            log.debug("遷移元画面", super.getInitialBackPage());
            log.debug("============================================================");

            if (super.isInitialize()) {

                log.debug("------------------------------------------------------------");
                log.debug("1.初期処理");
                log.debug("1-1.法人一覧の取得(システム管理者は全件／その他は操作中の法人)");
                log.debug("------------------------------------------------------------");

                // ルールエンジン:初期処理
                final RuleEngineIF initializeRule = super.getRuleEngine("Initialize");

                initializeRule.run();

                log.debug("------------------------------------------------------------");
                log.debug("2.初期検索");
                log.debug("------------------------------------------------------------");

                // ユーザー情報
                final UserInfo info = UserInfo.getUserInfo();

                super.setString("SEARCH_CORPORATE_CODE", info.getCorporateCode());
                super.setDate("SEARCH_F_DATE", new Date(System.currentTimeMillis()));

                this.btn_search_event();
            }
        }
    }

    /**
     * 休日(定休日/休業日/祝日・休日)の検索処理。
     * <br>1.入力された条件で休日設定を検索
     * <br>2.画面を再描画
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_search_event()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("1.入力された条件で休日設定を検索");
        log.debug("============================================================");

        // ルールエンジン:検索処理
        final RuleEngineIF findRule = super.getRuleEngine("Find");

        findRule.run();

        log.debug("============================================================");
        log.debug("2.画面を再描画");
        log.debug("============================================================");
        this.rewrite("一覧再描画");
    }

    /**
     * 定休日設定画面へ遷移。
     * <br>1.定休日設定画面(SYS1300D)へ遷移
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_create_event()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("1.定休日設定画面(SYS1300D)へ遷移");
        log.debug("============================================================");

        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        // 定休日設定画面
        final EventIF e = new SYS1300DEvent();

        if (info.hasAuthority("SYSTEM_ADMIN")) {
            e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));
        } else {
            e.setString("CORPORATE_CODE", info.getCorporateCode());
        }
        super.setNextPage(e);
    }

    /**
     * 休業日の登録処理に係る入力チェック。
     * <br>1.画面の入力値の妥当性をチェック
     * <br>1-1.必須入力チェック
     * <br>1-2.重複チェック
     * @return true 正常 or false 異常
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("1.画面の入力値の妥当性をチェック");
        log.debug("1-1.必須入力チェック");
        log.debug("1-2.重複チェック");
        log.debug("============================================================");

        // ルールエンジン:休業日の登録処理
        final RuleEngineIF createHolidayRule = super.getRuleEngine("CreateHoliday");

        return createHolidayRule.check();
    }

    /**
     * 休業日の登録処理。
     * <br>1.画面で入力された休業日を登録
     * <br>2.検索結果を最新化して再描画
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("1.画面で入力された休業日を登録");
        log.debug("============================================================");

        // ルールエンジン:休業日の登録処理
        final RuleEngineIF createHolidayRule = super.getRuleEngine("CreateHoliday");

        createHolidayRule.run();

        log.debug("============================================================");
        log.debug("2.検索結果を最新化して再描画");
        log.debug("============================================================");

        // 検索結果一覧
        final PaginationBean list
         = (PaginationBean)super.getSubList("SEARCH_RESULT");

        if (list != null) list.refresh();

        this.rewrite("一覧再描画");
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearRewrite();

        log.debug(pmMode);

        switch (pmMode)
        {
            case "一覧再描画":
            {
                ctrl.addRewrite("BLOCK_SEARCH_RESULT");
                break;
            }
        }
    }


    /**
     * サブイベント:休日。
     * <br>休日設定画面の休日の検索結果一覧の１行データ
     * <br>(A)行データの削除
     * <br>1.当該データの削除
     * <br>2.画面再描画
     * <br>◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆◇◆
     */
    public static class Holiday
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            // ログAPI
            final DebuggerIF log = DebuggerFactory.get();

            if (pmCmd.endsWith("$BTN_DELETE")) {

                log.debug("============================================================");
                log.debug("(A)行データの削除");
                log.debug("============================================================");

                log.debug("------------------------------------------------------------");
                log.debug("1.当該データの削除");
                log.debug("------------------------------------------------------------");

                // 親イベント
                final SYS1300SEvent parent = (SYS1300SEvent)super.getParentEvent();

                // ルールエンジン:指定休日の削除
                final RuleEngineIF deleteHolidayRule = super.getRuleEngine("DeleteHoliday");

                if (deleteHolidayRule.check()) {

                    log.debug("入力チェック：正常 - 処理実行");
                    deleteHolidayRule.run();

                } else {

                    log.debug("処理中断：入力チェック・エラー");
                    return false;
                }

                if (super.getInteger("DELETE_RESULT") == 0) {
                    log.debug("処理中断：排他制御エラー");
                    return false;
                }

                log.debug("------------------------------------------------------------");
                log.debug("2.画面再描画");
                log.debug("------------------------------------------------------------");
                super.setString("FLAG", "-1");
                parent.rewrite("一覧再描画");

            } else {
               return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
        {

        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public Holiday(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public Holiday()
            throws RuleException
        {
            super();
        }
    }
}
