package jp.co.hogehoge.system.help;

import java.sql.Date;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

// commons
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.util.text.format.datetime.DateTimeFactory;
import jp.co.cam.usax.util.text.format.datetime.DateTimeIF;
import jp.co.cam.usax.util.text.format.datetime.festival.FestivalManagerIF.FestivalIF;
import jp.co.cam.usax.USAXException;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.bean.DataBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF.SubListBeanIF;
import jp.co.cam.usax.rule.bean.impl.ListBean;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.WebAnnotations.PageScope;
import jp.co.cam.usax.web.event.EventIF;
import jp.co.cam.usax.web.event.io.ScreenControllerIF;

// example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;


/**
 * 【Example】日付入力補助画面(カレンダー)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@PageScope("request")
@Transaction({"default"})
public class HELP0000SEvent
    extends AbstractExampleScreenEvent
{
    /** 週一覧 */
    public static final String[] WEEKS = new String[]
    {
        "MON"
       ,"TUE"
       ,"WED"
       ,"THU"
       ,"FRI"
       ,"SAT"
       ,"SUN"
    };

    // パラメータ ==================================================================

    /** パラメータ：法人コード */
    public static final String PRM_CORPORATE_CODE = "SEARCH_CORPORATE_CODE";

    /** パラメータ：種別 */
    public static final String PRM_CALENDAR_CODE = "SEARCH_CALENDAR_CODE";

    /** パラメータ：日付 */
    public static final String PRM_DATE = "DATE";


    // 返却先 ====================================================================

    /** 返却先：日付 */
    public static final String RES_DATE = "RE_DATE";

    /** 返却先：曜日 */
    public static final String RES_WEEK = "RE_WEEK";

    /** 返却先：年 */
    public static final String RES_YYYY = "RE_YYYY";

    /** 返却先：月 */
    public static final String RES_MM   = "RE_MM";

    /** 返却先：日 */
    public static final String RES_DD   = "RE_DD";


    /** SerialVersionUID */
    private static final long serialVersionUID = -2017L;


    @Override
    public void page_load()
        throws RuleException
    {
        try {

            // ユーザー情報
            final UserInfo info = UserInfo.getUserInfo();

            // 日時API
            final DateTimeIF format = DateTimeFactory.get(info.getLanguage(), info.getCountry());
            
            // カレンダー
            final Calendar calendar = format.getCalendarInstance();

            // 月
            final Integer month = super.getInteger("MONTH");

            // 年
            final Integer year  = super.getInteger("YEAR");

            if (month != null && year != null) {
                calendar.set(Calendar.MONTH, month - 1);
                calendar.set(Calendar.YEAR,  year);
            }

            calendar.set(Calendar.HOUR_OF_DAY,  0);
            calendar.set(Calendar.MINUTE,       0);
            calendar.set(Calendar.SECOND,       0);
            calendar.set(Calendar.MILLISECOND,  0);
            calendar.set(Calendar.DAY_OF_MONTH, 1);

            // ====================================================================================
            // 初期処理
            // ====================================================================================
            if (!super.isPostBack()) {
    
                if (super.isInitialize()) {

                    // ----------------------------------------------------------------------------
                    // QueryStringの読込み
                    // ----------------------------------------------------------------------------

                    // QueryString
                    final Map<String, String> parameter = super.getUserIO().getQueryStringMap();

                    // 法人一覧
                    final ListBeanIF corporateList = new ListBean(this);

                    SubListBeanIF corporate;

                    // 法人ID
                    final String corporateCode = parameter.get(PRM_CORPORATE_CODE);

                    if (corporateCode != null && corporateCode.length() != 0) {
                    // 法人の指定あり

                        if (!corporateCode.equals(info.getCorporateCode())) {
                            corporate = corporateList.createSubBean();
                            corporate.setString("SELECTED", corporateCode);
                            corporateList.add(corporate);
                        }
                    }

                    // 操作法人
                    corporate = corporateList.createSubBean();
                    corporate.setString("SELECTED", info.getCorporateCode());
                    corporateList.add(corporate);
                    super.setSubList("CORPORATE_LIST", corporateList);

                    super.setString(PRM_CORPORATE_CODE, corporateCode);

                    super.setString(PRM_CALENDAR_CODE, parameter.get(PRM_CALENDAR_CODE));
                    super.setString(PRM_DATE, parameter.get(PRM_DATE));

                    super.setString(RES_DATE, parameter.get(RES_DATE));
                    super.setString(RES_WEEK, parameter.get(RES_WEEK));
                    super.setString(RES_YYYY, parameter.get(RES_YYYY));
                    super.setString(RES_MM,   parameter.get(RES_MM));
                    super.setString(RES_DD,   parameter.get(RES_DD));

                    // ----------------------------------------------------------------------------
                    // 現在日付を設定
                    // ----------------------------------------------------------------------------

                    // 現在日付
                    final Date today = new Date(System.currentTimeMillis());
    
                    super.setDate("TODAY", today);
                    super.setDate("WEEK",  today);

                    // 初期データの取得
                    super.getRuleEngine("Initialize").run();

                    // ====================================================================================
                    // 年月の選択肢を作成
                    // ====================================================================================
                    if (super.getString("DATE") != null) {
                    // 初期日付あり

                        calendar.setTime(super.getDate("DATE"));
                        calendar.set(Calendar.HOUR_OF_DAY,  0);
                        calendar.set(Calendar.MINUTE,       0);
                        calendar.set(Calendar.SECOND,       0);
                        calendar.set(Calendar.MILLISECOND,  0);
                        calendar.set(Calendar.DAY_OF_MONTH, 1);

                        super.setString("DATE", null);
                    }

                    super.setInteger("MONTH", calendar.get(Calendar.MONTH) + 1);
                    super.setInteger("YEAR",  calendar.get(Calendar.YEAR));

                    // 初期検索の実行
                    this.btn_search_event();
                }
            }

            this.initializeYearList(calendar);

        } catch (final USAXException ex) {
            throw new RuleException(ex, "カレンダーの初期化に失敗");
        }
    }


    /**
     * 前月処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_date_back_event()
        throws RuleException
    {
        try {

            // 月
            int month = super.getInteger("MONTH");
    
            // 年
            int year = super.getInteger("YEAR");
    
            if (month ==  1) {
                super.setInteger("YEAR",  year - 1);
                super.setInteger("MONTH", 12);
            } else {
                super.setInteger("MONTH", month - 1);
            }
            this.btn_search_event();

        } catch (final RuleException ex) {
            throw new RuleException(ex, "前月への切替えに失敗");
        }
    }

    /**
     * 次月処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_date_next_event()
        throws RuleException
    {
        try {
    
            // 月
            int month = super.getInteger("MONTH");
    
            // 年
            int year = super.getInteger("YEAR");
    
            if (month == 12) {
                super.setInteger("YEAR",  year + 1);
                super.setInteger("MONTH", 1);
            } else {
                super.setInteger("MONTH", month + 1);
            }
            this.btn_search_event();

        } catch (final RuleException ex) {
            throw new RuleException(ex, "次月への切替えに失敗");
        }
    }


    /**
     * 検索処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_search_event()
        throws RuleException
    {
        try {

            this.createMonthlyData(3);

        } catch (final USAXException ex) {
            throw new RuleException(ex, "カレンダーの生成に失敗");
        }

        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        // 日時API
        final DateTimeIF dateTime;

        // 開始日
        java.util.Date dateFrom;

        // 終了日
        java.util.Date dateTo;

        try {

            dateTime = DateTimeFactory.get(info.getLanguage(), info.getCountry());

            dateFrom = dateTime.getFirstDateTimeOfMonth(super.getDate("DATE0"));
            super.setDate("SEARCH_F_DATE", new Date(dateFrom.getTime()));

            dateTo = dateTime.getLastDateTimeOfMonth(super.getDate("DATE2"));
            super.setDate("SEARCH_T_DATE", new Date(dateTo.getTime()));

            super.getRuleEngine("GetHoliday").run();

        } catch (final USAXException ex) {
            throw new RuleException(ex, "休日情報の取得に失敗");
        }
        this.rewrite("一覧再描画");
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearRewrite();

        switch (pmMode)
        {
            case "一覧再描画":

                // 入力補助の返却イベントを設定
                ctrl.addRewrite("HELP_RESULT");
                break;
        }
    }


    /**
     * カレンダー情報の生成。
     * <br>指定個数分のカレンダー情報を作成しインスタンスに保持する。
     * @param  pmCount 個数
     * @throws RuleException カレンダー情報の生成に失敗した場合
     */
    private void createMonthlyData(
        final int pmCount
    )
        throws USAXException
    {
        // 本日
        final Date today = super.getDate("Today");

        // １週間
        ListBeanIF.SubListBeanIF weekly;

        // １カ月
        ListBeanIF monthly;

        // 月
        int month;

        // 日
        int day;

        // 行
        int row;

        // カレンダー
        Calendar calendar;

        // 日付
        Date dat;

        // 曜日
        int weekIndex;

        // 祝日・休日一覧
        List<FestivalIF> holidayList;

        // 祝日・休日
        java.util.Date holiday;

        // ========================================================================================
        // 指定数分のカレンダー情報を生成
        // ========================================================================================
        for (int index = 0; index < pmCount; index++)
        {
            // ------------------------------------------------------------------------------------
            // ６週分のデータ・コンテナの生成
            // ------------------------------------------------------------------------------------
            monthly = new ListBean(this);
            monthly.setSubClass(Week.class);

            for (int week = 0; week < 6; week++) monthly.add(monthly.createSubBean());

            // ------------------------------------------------------------------------------------
            // 日付データの設定
            // ------------------------------------------------------------------------------------
            calendar = this.initializeCalendar(index);

            holidayList = this.getHoliday(calendar.get(Calendar.YEAR));

            month = calendar.get(Calendar.MONTH);
            day   = calendar.get(Calendar.DAY_OF_WEEK);
            row   = 0;

            super.setDate("DATE" + index, new Date(calendar.getTime().getTime()));

            for (int i = day; i <= (7 * 6); i++)
            {
                weekly = monthly.get(row);
    
                dat = new Date(calendar.getTimeInMillis());

                switch (calendar.get(Calendar.DAY_OF_WEEK))
                {
                    case Calendar.MONDAY:
                    {
                        weekIndex = 0;
                        break;
                    }
                    case Calendar.TUESDAY:
                    {
                        weekIndex = 1;
                        break;
                    }
                    case Calendar.WEDNESDAY:
                    {
                        weekIndex = 2;
                        break;
                    }
                    case Calendar.THURSDAY:
                    {
                        weekIndex = 3;
                        break;
                    }
                    case Calendar.FRIDAY:
                    {
                        weekIndex = 4;
                        break;
                    }
                    case Calendar.SATURDAY:
                    {
                        weekIndex = 5;
                        break;
                    }
                    case Calendar.SUNDAY:
                    {
                        weekIndex = 6;
                        break;
                    }
                    default:
                        weekIndex = -1;
                }

                if (today.toString().equals(dat.toString())) {
                    weekly.setString("TODAY_WEEK", WEEKS[weekIndex]);
                }
                weekly.setDate("DATE_" + WEEKS[weekIndex], dat);
                weekly.setDate("WEEK_" + WEEKS[weekIndex], dat);
                weekly.setDate("YYYY_" + WEEKS[weekIndex], dat);
                weekly.setDate("MM_"   + WEEKS[weekIndex], dat);
                weekly.setDate("DD_"   + WEEKS[weekIndex], dat);

                // ----------------------------------------------------------------------
                // 祝日判定
                // ----------------------------------------------------------------------
                for (final FestivalIF tmp : holidayList)
                {
                    // 祝日の確認
                    holiday = tmp.getFestivalDate();

                    if (holiday != null && holiday.compareTo(dat) == 0) {
                        weekly.setBoolean("HOLIDAY_" + WEEKS[weekIndex], true);
                        break;
                    }

                    // 振替の確認
                    holiday = tmp.getSubstitute();

                    if (holiday != null && holiday.compareTo(dat) == 0) {
                        weekly.setBoolean("HOLIDAY_" + WEEKS[weekIndex], true);
                        break;
                    }
                }

                // ----------------------------------------------------------------------
                // 翌日へ
                // ----------------------------------------------------------------------
                calendar.add(Calendar.DAY_OF_MONTH, 1);

                if (month == calendar.get(Calendar.MONTH)) {
                // 同月
    
                    day++;
    
                    if (7 < day) {
                        day = 1;
                        row++;
                    }
    
                } else {
                // 翌月(終了)

                    break;
                }
            }
            super.setSubList("CALENDAR" + index, monthly);
        }
    }

    /**
     * カレンダーAPIを生成する。
     * @param  pmIndex インデックス
     * @return Calendar
     * @throws RuleException 生成時に発生した例外を報告
     */
    private Calendar initializeCalendar(
        final int pmIndex
    )
        throws RuleException
    {
        try {

            // 日付様式
            final DateTimeIF format = DateTimeFactory.get();

            // カレンダー
            final Calendar calendar = format.getCalendarInstance();

            // ------------------------------------------------------------------------------------
            // 日時をクリア
            // ------------------------------------------------------------------------------------
            calendar.set(Calendar.MONTH, super.getInteger("MONTH") - 1);
            calendar.set(Calendar.YEAR,  super.getInteger("YEAR"));
            calendar.set(Calendar.HOUR_OF_DAY,  0);
            calendar.set(Calendar.MINUTE,       0);
            calendar.set(Calendar.SECOND,       0);
            calendar.set(Calendar.MILLISECOND,  0);
            calendar.set(Calendar.DAY_OF_MONTH, 1);

            calendar.add(Calendar.MONTH, pmIndex);

            return calendar;

        } catch (final Exception ex) {
            throw new RuleException(ex, "日付様式APIの生成に失敗");
        }
    }

    /**
     * 年一覧を生成する。
     * @param  pmCalendar カレンダー
     * @throws RuleException 生成時に発生した例外を報告
     */
    private void initializeYearList(
        final Calendar pmCalendar
    )
        throws RuleException
    {
        // 年一覧
        final ListBeanIF yearList = new ListBean(this);

        // 年
        SubListBeanIF year;

        // [仕様]前後５年分の一覧を生成する
        for (int i = (pmCalendar.get(Calendar.YEAR) - 5); i < (pmCalendar.get(Calendar.YEAR) + 5); i++)
        {
            year = yearList.createSubBean();
            year.setInteger("Year", i);
            yearList.add(year);
        }
        super.setSubList("YearList", yearList);
    }

    /**
     * 指定年における祝日一覧を取得する。
     * @param  pmYear 対象の年
     * @return 祝日一覧
     * @throws RuleException 祝日データの取得に失敗した場合
     */
    private List<FestivalIF> getHoliday(
        final int pmYear
    )
        throws RuleException
    {
        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        try {

            return DateTimeFactory.get(info.getLanguage(), info.getCountry()).getFestivalManager().getFestivalList(pmYear);

        } catch (final Exception ex) {
            throw new RuleException(ex, "祝日データの取得に失敗");
        }
    }


    /**
     * 1週間分のデータ。
     */
    public static class Week
        extends CommonExampleSubEvent
    {
        /** Serial Version */
        private static final long serialVersionUID = -2020L;


        public boolean isNotWorkingDay(
            final String pmKey
        )
            throws RuleException
        {
            // 親イベント
            final EventIF parent = super.getParentEvent();

            // 日
            final String date = super.getString(pmKey);

            if (date != null) {

                // 休日情報
                final DataBeanIF holidayData = parent.getSubBean("HOLIDAY");

                return (holidayData.getString(date) != null);

            } else {
                return false;
            }
        }

        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            // 親イベント
            final EventIF parent = super.getParentEvent();

            // 遷移元
            final EventIF opener = parent.getInitialBackPage();

            // 日付
            final String reDate = parent.getString("RE_DATE");

            // 曜日
            final String reWeek = parent.getString("RE_WEEK");

            // 年
            final String reY = parent.getString("RE_Y");

            // 月
            final String reM = parent.getString("RE_M");

            // 日
            final String reD = parent.getString("RE_D");

            String key;

            if (pmCmd.contains("$BTN_SELECT_")) {

                key = pmCmd.substring(pmCmd.indexOf("$BTN_SELECT_") + "$BTN_SELECT_".length());

                if (reDate != null) {
                    opener.setString(reDate, super.getString("DATE_" + key));
                }
                if (reWeek != null) {
                    opener.setString(reWeek, super.getString("WEEK_" + key));
                }
                if (reY != null) {
                    opener.setString(reY, super.getString("YYYY_" + key));
                }
                if (reM != null) {
                    opener.setString(reM, super.getString("MM_" + key));
                }
                if (reD != null) {
                    opener.setString(reD, super.getString("DD_" + key));
                }

                parent.setNextPage(opener);

                return true;
            }
            return super.data_command(pmCmd);
        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public Week(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public Week()
            throws RuleException
        {
            super();
        }
    }
}
